# Copyright (c) 2014  Dave Love, University of Liverpool
# Licence as for the package itself

# fixme: --with-64-bit-indices version
#        java, papi?, numpy?, sundials?
#        disable calling home on build

# MUMPS-mpich isn't built in Fedora
%bcond_with mpich

# use rebuilt v4 version in EPEL6
%if 0%{?el6}
%bcond_with suitesparse
%else
%bcond_without suitesparse
%endif

# Turn off X to be friendlier for compute nodes.
# Other components: fftw needs mpi version; sprng apparently needs sprng1;
#   tetgen seems to have an incompatible interface

Name:           petsc
Summary:        Portable Extensible Toolkit for Scientific Computation
License:        MIT
Version:        3.5.2
Release:        4%{?dist}

Source:         ftp://ftp.mcs.anl.gov/pub/petsc/release-snapshots/petsc-%{version}.tar.gz
# Patches from OpenSuSE version
Patch1:         petsc-3.3-p2-no-rpath.patch
URL:            http://www.mcs.anl.gov/petsc/
BuildRoot:      %{_tmppath}/%{name}-%{version}-build
BuildRequires:  gcc-c++ gcc-gfortran
BuildRequires:  openblas-devel openssl-devel zlib-devel
BuildRequires:  blacs-openmpi-devel scalapack-openmpi-devel
BuildRequires:  hdf5-openmpi-devel metis-devel SuperLU-devel
BuildRequires:  hypre-openmpi-devel hypre-devel scotch-devel
BuildRequires:  openmpi-devel cgnslib-devel valgrind-devel
BuildRequires:  ptscotch-openmpi-devel MUMPS-openmpi-devel MUMPS-devel
# fixme: Doesn't work with 3.4 (RHEL 6), but don't know which version is
# actually required.
%if %{with suitesparse}
BuildRequires:  suitesparse-devel >= 4.0
%endif
# non-free
#BuildRequires: 	triangle-devel
%if %{with mpich}
BuildRequires:  mpich-devel hdf5-mpich-devel hypre-mpich-devel
BuildRequires:	blacs-mpich-devel scalapack-mpich-devel
BuildRequires:  ptscotch-mpich-devel MUMPS-mpich-devel MUMPS-devel
%endif


%global desc \
PETSc is the "Portable Extensible Toolkit for Scientific Computation",\
a suite of data structures and routines for the scalable (parallel)\
solution of scientific applications modeled by partial differential\
equations.  It employs the MPI standard for all message-passing\
communication.  Several sample scientific applications, as well as\
various papers and talks, demonstrate the features of the PETSc\
libraries.

%description
%desc

This is the serial version.

%package devel
Summary:        Development files for %name
Requires:       %name = %{version}-%{release}
Requires:       hdf5-devel%{_isa} metis-devel%{_isa} SuperLU-devel%{_isa}
Requires:       cgnslib-devel%{_isa} zlib-devel%{_isa} scotch-devel%{_isa}
Requires:       MUMPS-devel%{_isa}
%if %{with suitesparse}
Requires:       suitesparse-devel%{_isa} >= 4.0
%endif

%description devel
PETSc is a suite of data structures and routines for the scalable
(parallel) solution of scientific applications modeled by partial 
differential equations.

%package openmpi
Summary:        Portable Extensible Toolkit for Scientific Computation - openmpi
Requires:       openmpi

%description openmpi
%desc

This the the openmpi version.

%package openmpi-devel
Summary:        Development files for %name-openmpi
Requires:       %name-openmpi%{_isa} = %{version}-%{release}
Requires:       blacs-openmpi-devel%{_isa} scalapack-openmpi-devel%{_isa}
Requires:       hdf5-openmpi-devel%{_isa} cgnslib-devel%{_isa}
Requires:       hypre-openmpi-devel%{_isa} metis-devel%{_isa}
Requires:       ptscotch-openmpi-devel%{_isa} MUMPS-openmpi-devel%{_isa}
%if %{with suitesparse}
Requires:       suitesparse-devel%{_isa} >= 4.0
%endif

%description openmpi-devel
Development files for %name-openmpi.

%if %{with mpich}
%package mpich
Summary:        Portable Extensible Toolkit for Scientific Computation - mpich
Requires:       mpich


%description mpich
%desc

This the the mpich version.

%package mpich-devel
Summary:        Development files for %name-mpich
Requires:       blacs-mpich-devel%{_isa} scalapack-mpich-devel%{_isa}
Requires:       hdf5-mpich-devel%{_isa} hypre-mpich-devel%{_isa}
Requires:       %name-mpich%{_isa} = %{version}-%{release}
%if %{with suitesparse}
Requires:       suitesparse-devel%{_isa} >= 4.0
%endif

%description mpich-devel
Development files for %name-mpich.
%endif

%package doc
Summary:        Documentation for %name
BuildArch:      noarch

%description doc
Documentation for %name.

%prep
%setup -q -c -n petsc-%{version}

pushd petsc-%{version}
rm -r bin/win32fe               # MS Windows binary
%patch1 -p1 -b .rpath
popd
%if %{with mpich}
cp -a petsc-%{version} mpich
%endif
cp -a petsc-%{version} openmpi


%build

%if %{with suitesparse}
%global suitesparse --with-suitesparse=1 --with-suitesparse-include=%{_includedir}/suitesparse --with-suitesparse-lib=[%_libdir/libumfpack.so,%_libdir/libklu.so,%_libdir/libcholmod.so,%_libdir/libbtf.so,%_libdir/libccolamd.so,%_libdir/libcolamd.so,%_libdir/libcamd.so,%_libdir/libamd.so,%_libdir/libsuitesparseconfig.so]
%endif

pushd petsc-%{version}
# The increased optimization level is a guess -- effects not measured --
# but loop unrolling is likely to worthwhile for the Fortran.
PETSC_ARCH=arch-linux2-c-debug \
%configure \
  CFLAGS="$RPM_OPT_FLAGS -O3" FFLAGS="$RPM_OPT_FLAGS -O3 -funroll-loops" \
  CXXFLAGS="$RPM_OPT_FLAGS" LDFLAGS="$LDFLAGS" \
  --prefix=/usr \
  --includedir=%_includedir/%name \
  --with-fortran-interfaces=1 --with-x=0 \
  --with-debugging=1 --with-shared-libraries --with-mpi=0 \
  --with-batch=0 \
  --with-cgns=1 --with-cgns-dir=/usr \
  --with-blas-lapack-lib=%_libdir/libopenblas.so \
  --with-superlu=1 --with-superlu-lib=%_libdir/libsuperlu.so \
  --with-superlu-include=%_includedir/SuperLU \
  --with-metis=1 --with-metis-dir=/usr \
  --with-valgrind=1 --with-valgrind-dir=/usr \
  %suitesparse \
  --with-blas-lapack-lib=%_libdir/libopenblas.so

make V=1                       # rejects -j ("automatically parallel")
popd

%global do_build \
PETSC_ARCH=arch-linux2-c-debug \\\
%configure \\\
  --with-fc=mpif90 --with-cc=mpicc --with-cxx=mpicxx \\\
  CFLAGS="$RPM_OPT_FLAGS -O3" FFLAGS="$RPM_OPT_FLAGS -O3 -funroll-loops" \\\
  CXXFLAGS="$RPM_OPT_FLAGS" LDFLAGS="$LDFLAGS" \\\
  --with-debugging=1 --with-x=0 --prefix=$MPI_HOME \\\
  --with-fortran-interfaces=1 \\\
  --with-shared-libraries --with-mpi=1 --with-make-np=$RPM_BUILD_NCPUS \\\
  --with-batch=0 \\\
  --with-blas-lapack-lib=%_libdir/libopenblas.so \\\
  --with-scalapack=1 \\\
  --with-scalapack-lib=[$MPI_LIB/libscalapack.so,$MPI,$MPI_LIB/libmpiblacs.so] \\\
  --with-hypre=1 --with-hypre-include=$MPI_INCLUDE/hypre --with-hypre-lib=[$MPI_LIB/libHYPRE.so,$MPI_LIB/libHYPRE_IJ_mv.so] \\\
  --with-superlu=1 --with-superlu-lib=%_libdir/libsuperlu.so \\\
  --with-superlu-include=%_includedir/SuperLU \\\
  --with-hdf5=1 --with-hdf5-dir=/usr \\\
  --with-ptscotch=1 --with-ptscotch-dir=$MPI_HOME \\\
  --with-mumps=1 --with-mumps-dir=$MPI_HOME \\\
  --with-cgns=1 --with-cgns-dir=/usr \\\
  --with-metis=1 --with-metis-dir=/usr \\\
  %suitesparse \\\
  --with-valgrind=1 --with-valgrind-dir=/usr \
\
make V=1

pushd openmpi
%_openmpi_load
%do_build
%_openmpi_unload
popd

%if %{with mpich}
pushd mpich
%_mpich_load
%do_build
%_mpich_unload
popd
%endif


%install

pushd %name-%version
mkdir -p $RPM_BUILD_ROOT%_libdir $RPM_BUILD_ROOT%_datadir/%name
make install DESTDIR=$RPM_BUILD_ROOT/usr
mv $RPM_BUILD_ROOT/usr/lib/*.so* $RPM_BUILD_ROOT%_libdir
mv $RPM_BUILD_ROOT/usr/lib/pkgconfig $RPM_BUILD_ROOT%_libdir
rm $RPM_BUILD_ROOT/usr/conf/*.{log,init,py}
mv $RPM_BUILD_ROOT/usr/conf $RPM_BUILD_ROOT%_datadir/%name
mkdir $RPM_BUILD_ROOT%_includedir/.petsc
mv $RPM_BUILD_ROOT%_includedir/* $RPM_BUILD_ROOT%_includedir/.petsc
mv $RPM_BUILD_ROOT%_includedir/.petsc $RPM_BUILD_ROOT%_includedir/petsc
rm -r $RPM_BUILD_ROOT%_bindir $RPM_BUILD_ROOT/usr/lib/.DIR
find $RPM_BUILD_ROOT%_includedir/ -name \*html -exec rm {} \;
sed -i 's/set petsc_arch *$/set petsc_arch ""/' $RPM_BUILD_ROOT/usr/lib/modules/*
# having /usr/bin here loses on unloading
sed -i '/PATH/d' $RPM_BUILD_ROOT/usr/lib/modules/*
mkdir -p $RPM_BUILD_ROOT%_sysconfdir/modulefiles
mv $RPM_BUILD_ROOT/usr/lib/modules $RPM_BUILD_ROOT%_sysconfdir/modulefiles/petsc
popd

mkdir -p $RPM_BUILD_ROOT/%_includedir/{openmpi,mpich}-%_arch
for mpi in openmpi \
%if %{with mpich}
mpich \
%endif
; do
  pushd $mpi
  case $mpi in
  openmpi) %_openmpi_load ;;
  mpich) %_mpich_load ;;
  esac
  make install DESTDIR=$RPM_BUILD_ROOT$MPI_HOME
  mkdir -p $RPM_BUILD_ROOT$MPI_HOME/share/%name
  rm $RPM_BUILD_ROOT$MPI_HOME/conf/*.{log,init,py}
  mv $RPM_BUILD_ROOT$MPI_HOME/conf $RPM_BUILD_ROOT$MPI_HOME/share/%name
  rm -r $RPM_BUILD_ROOT$MPI_HOME/bin $RPM_BUILD_ROOT$MPI_LIB/.DIR
  # fix invalid tcl
  sed -i 's/set petsc_arch *$/set petsc_arch ""/' $RPM_BUILD_ROOT$MPI_LIB/modules/*
  mkdir -p $RPM_BUILD_ROOT%_sysconfdir/modulefiles
  mv $RPM_BUILD_ROOT$MPI_LIB/modules $RPM_BUILD_ROOT%_sysconfdir/modulefiles/petsc-$mpi
  find $RPM_BUILD_ROOT$MPI_HOME/include/ -name \*html -exec rm {} \;
  mkdir -p $MPI_INCLUDE
  mv $RPM_BUILD_ROOT$MPI_HOME/include $RPM_BUILD_ROOT$MPI_INCLUDE/petsc
  case $mpi in
  openmpi) %_openmpi_unload ;;
  mpich) %_mpich_unload ;;
  esac
  popd
done

find $RPM_BUILD_ROOT%_datadir/%name $RPM_BUILD_ROOT%_libdir/*/share -name \*.html -exec rm {} \;
find $RPM_BUILD_ROOT%_datadir/%name $RPM_BUILD_ROOT%_libdir/*/share -type f -exec chmod 644 {} \;
find $RPM_BUILD_ROOT%_libdir -name libpetsc.so.\* -exec chmod 0755 {} \;


%check
%_openmpi_load
OMPI_MCA_orte_allocation_required=0 make -C openmpi test
# fails to build (because not properly installed?)
#OMPI_MCA_orte_allocation_required=0 make NPMAX=2 streams
%_openmpi_unload


%clean
rm -rf $RPM_BUILD_ROOT

%post -p /sbin/ldconfig
%postun -p /sbin/ldconfig

%files
%doc %name-%version/LICENSE
%{_libdir}/*.so.*
%_datadir/%name
%_sysconfdir/modulefiles/petsc

%files devel
%_includedir/%name
%{_libdir}/*.so
%{_libdir}/pkgconfig/*

%files openmpi
%doc openmpi/LICENSE
%_libdir/openmpi/lib/*.so.*
%{_libdir}/openmpi/share/%name
%_sysconfdir/modulefiles/petsc-openmpi

%files openmpi-devel
%doc openmpi/CONTRIBUTING
%_includedir/openmpi-%_arch/%name
%_libdir/openmpi/lib/pkgconfig/*
%_libdir/openmpi/lib/*.so

%if %{with mpich}
%files mpich
%doc LICENSE
%_libdir/mpich/lib/*.so.*
%_sysconfdir/modulefiles/petsc-mpich

%files mpich-devel
%doc CONTRIBUTING
%_includedir/mpich-%_arch/%name
%_libdir/mpich/lib/pkgconfig
%_libdir/mpich/lib/*.so
%endif

%files doc
%doc openmpi/docs/* openmpi/tutorials


%changelog
* Sat Jan 17 2015 Dave Love <d.love@liverpool.ac.uk> - 3.5.2-4
- Fix serial module file

* Thu Jan  1 2015 Dave Love <d.love@liverpool.ac.uk> - 3.5.2-3
- Bump release because of copr problem

* Sat Dec 27 2014 Dave Love <d.love@liverpool.ac.uk> - 3.5.2-2
- Replace SuperLU after hypre modification
- Install module files

* Sun Dec 21 2014 Dave Love <d.love@liverpool.ac.uk> - 3.5.2-1
- New version

* Mon Dec 15 2014 Dave Love <d.love@liverpool.ac.uk> - 3.5.0-3
- Use valgrind, which build calls "highly recommended"

* Sun Dec 14 2014 Dave Love <d.love@liverpool.ac.uk> - 3.5.0-2
- Don't build against (non-free) triangle
- Conditional mpich build

* Tue Jul 15 2014 Dave Love <d.love@liverpool.ac.uk> - 3.5.0-1
- Initial packaging not based on OpenSuSE
